/* SPRpc - Copyright (C) 2008 Andrea Marchesini <baku@autistici.org>
 *                            Rocco De Patto    <lesion@autistici.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef __SP_RPC_INTERNAL_H__
#define __SP_RPC_INTERNAL_H__

#include "sp_rpc.h"
#include "../json/tb_json.h"

#define SP_RPC_RUN_TIMEOUT		500

struct sp_rpc_t
{
  /* Reference counter: */
  gint		ref;

  /* Loop indicator: */
  gboolean	is_running;

  /* Spread Mail: */
  mailbox	mailbox;

  /* The Group and the ID: */
  gchar *	group;
  gchar *	id;

  /* list of SPRpcCb: */
  GList *	methods;
  guint		method_last_numb;

  /* list of SPRpcCb: */
  GList *	responses;
  guint		response_last_numb;
};

typedef struct sp_rpc_cb_t SPRpcCb;
struct sp_rpc_cb_t
{
  guint		number;

  gchar *	name;
  gchar *	description;

  SPRpcMethod	method_function;
  SPRpcResponse	response_function;
  gpointer	user_data;

  SPRpcParams *	params;
};

struct sp_rpc_value_t
{
  SPRpcTypeValue	type;
  guint			ref;

  union
    {
      gdouble		v_number;
      gboolean		v_boolean;
      gchar *		v_string;
  
      struct
        {
          GList *	array;
	  guint		array_length;

	} 		v_array;

      GHashTable *	v_struct;
    };
};

struct sp_rpc_params_t
{
  guint		ref;
  GHashTable *	hash;
};

typedef enum
{
  SP_RPC_JSON_CALL,
  SP_RPC_JSON_LIST,
  SP_RPC_JSON_EXISTS,
  SP_RPC_JSON_PARAMS,
  SP_RPC_JSON_RESPONSE
} SPRpcJsonType;

gboolean	sp_rpc_params_new_from_json_value
					(tb_json_value_t *	value,
					 SPRpcParams **		params);

SPRpcParams *	sp_rpc_params_new_for_call_list
					(GList *		methods);

SPRpcParams *	sp_rpc_params_new_for_call_exists
					(gchar *		method);

SPRpcParams *	sp_rpc_params_validate	(SPRpcParams *		model,
					 SPRpcParams *		data);

gchar *		sp_rpc_json_create	(const gchar *		id,
					 guint			progress_id,
					 SPRpcJsonType		type,
					 const gchar *		method_name,
					 SPRpcParams *		params,
					 gsize	*		size);

gboolean	sp_rpc_json_parse	(gchar *		buffer,
					 gsize			size,
					 gchar **		id,
					 guint *		progress_id,
					 SPRpcJsonType *	type,
					 gchar **		method_name,
					 SPRpcParams **		params);

#endif

/* EOF */
