/* SPRpc - Copyright (C) 2008 Andrea Marchesini <baku@autistici.org>
 *                            Rocco De Patto    <lesion@autistici.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <stdio.h>
#include <sp_rpc.h>

static void
function (SPRpc * rpc, SPRpcResponseID id, SPRpcParams * params,
	  gpointer user_data)
{
  if (!params)
    fprintf (stdout, "No params returned\n");

  else
    {
      SPRpcValue *value = sp_rpc_params_get (params, "date");

      if (value && sp_rpc_value_get_type (value) == SP_RPC_TYPE_STRING)
	{
	  gchar *string = sp_rpc_value_get_string (value);
	  fprintf (stderr, "Return value: %s\n", string);
	  g_free (string);
	}

      else
	fprintf (stdout, "Incompatible value\n");
    }

  sp_rpc_quit (rpc);
}

int
main (void)
{
  gchar group[MAX_GROUP_NAME];
  SPRpcMethodID id;
  mailbox mbox;
  SPRpc *rpc;

  if (SP_connect ("4803", NULL, 0, 0, &mbox, group) != ACCEPT_SESSION)
    {
      fprintf (stderr, "Error connecting to the Spread Ring\n");
      return 0;
    }

  SP_join (mbox, "TEST_GROUP");

  rpc = sp_rpc_new ();
  sp_rpc_set_mailbox (rpc, mbox);
  sp_rpc_set_group (rpc, "TEST_GROUP");
  sp_rpc_set_id (rpc, "test_sp_rpc_client");

  sp_rpc_call (rpc, "test_function", NULL, SP_RPC_REQ_TYPE_CALL, &id,
	       function, NULL);

  if (sp_rpc_run (rpc) != SP_RPC_STATUS_DONE)
    fprintf (stderr, "Error looping\n");

  sp_rpc_unref (rpc);

  SP_leave (mbox, "TEST_GROUP");
  SP_disconnect (mbox);
  return 0;
}
