/* SPRpc - Copyright (C) 2008 Andrea Marchesini <baku@autistici.org>
 *                            Rocco De Patto    <lesion@autistici.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <stdio.h>
#include <sp_rpc.h>

static gint numb = 5;

static void
function (SPRpc * rpc, SPRpcMethodID id, SPRpcParams * params,
	  SPRpcParams * return_params, gpointer user_data)
{
  SPRpcValue *value;
  gchar tmp[1024];
  time_t t;

  t = time (NULL);
  snprintf (tmp, sizeof (tmp), ctime (&t));

  value = sp_rpc_value_new ();
  sp_rpc_value_set_type (value, SP_RPC_TYPE_STRING);
  sp_rpc_value_set_string (value, tmp);

  sp_rpc_params_add (return_params, "date", value);
  sp_rpc_value_unref (value);

  if (!--numb)
    sp_rpc_quit (rpc);
}

int
main (void)
{
  gchar group[MAX_GROUP_NAME];
  mailbox mbox;
  SPRpc *rpc;

  if (SP_connect ("4803", NULL, 0, 0, &mbox, group) != ACCEPT_SESSION)
    {
      fprintf (stderr, "Error connecting to the Spread Ring\n");
      return 0;
    }

  SP_join (mbox, "TEST_GROUP");

  rpc = sp_rpc_new ();
  sp_rpc_set_mailbox (rpc, mbox);
  sp_rpc_set_group (rpc, "TEST_GROUP");
  sp_rpc_set_id (rpc, "test_sp_rpc_server");

  sp_rpc_register (rpc, "test_function", "bla bla bla...", NULL, function,
		   NULL);

  if (sp_rpc_run (rpc) != SP_RPC_STATUS_DONE)
    fprintf (stderr, "Error looping\n");

  sp_rpc_unref (rpc);

  SP_leave (mbox, "TEST_GROUP");
  SP_disconnect (mbox);
  return 0;
}
