/* SPRpc - Copyright (C) 2008 Andrea Marchesini <baku@autistici.org>
 *                            Rocco De Patto    <lesion@autistici.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <stdio.h>
#include <sp_rpc.h>

#define TEST_VALUE_MAX	50

static void do_set_value (SPRpcValue * value, gint rec);
static void do_set_array (SPRpcValue * value, gint numb, gint rec);
static void do_set_struct (SPRpcValue * value, gint rec);

static void do_get_value (SPRpcValue * value, gint rec);
static void do_get_array (SPRpcValue * value, gint rec);
static void do_get_struct (SPRpcValue * value, gint rec);

static void do_print (gint ret, gchar * str);

int
main (void)
{
  gint i;
  gint rec;
  SPRpcValue *value;

  rec = g_random_int_range (2, TEST_VALUE_MAX);
  value = sp_rpc_value_new ();

  for (i = 0; i < TEST_VALUE_MAX; i++)
    {
      do_set_value (value, rec);
      do_get_value (value, 0);
    }

  sp_rpc_value_unref (value);
  return 0;
}

static void
do_set_value (SPRpcValue * value, gint rec)
{
  gint type = g_random_int_range (SP_RPC_TYPE_NULL, SP_RPC_TYPE_STRUCT + 1);
  sp_rpc_value_set_type (value, type);

  switch (type)
    {
    case SP_RPC_TYPE_NULL:
      break;

    case SP_RPC_TYPE_NUMBER:
      sp_rpc_value_set_number (value,
			       g_random_double_range (-G_MAXDOUBLE,
						      G_MAXDOUBLE));
      break;

    case SP_RPC_TYPE_BOOLEAN:
      sp_rpc_value_set_boolean (value, g_random_int_range (FALSE, TRUE));
      break;

    case SP_RPC_TYPE_STRING:
      sp_rpc_value_set_string (value, "just a test");
      break;

    case SP_RPC_TYPE_ARRAY:
      do_set_array (value, g_random_int_range (1, rec),
		    g_random_int_range (1, rec));
      break;

    case SP_RPC_TYPE_STRUCT:
      do_set_struct (value, g_random_int_range (1, rec));
      break;
    }
}

static void
do_set_array (SPRpcValue * value, gint numb, gint rec)
{
  gint i;

  for (i = 0; i < rec; i++)
    {
      SPRpcValue *v;

      if ((i % 2))
	sp_rpc_value_array_set_new (value, &v);

      else
	{
	  v = sp_rpc_value_new ();
	  sp_rpc_value_array_set (value, v);
	  sp_rpc_value_unref (v);
	}

      if (rec > 2)
	do_set_value (v, rec - 1);
    }
}

static void
do_set_struct (SPRpcValue * value, gint rec)
{
  gint i;

  for (i = 0; i < rec; i++)
    {
      SPRpcValue *v;
      gchar tmp[1024];

      snprintf (tmp, sizeof (tmp), "key %d", i);

      if (i % 2)
	sp_rpc_value_struct_set_key_new (value, tmp, &v);

      else
	{
	  v = sp_rpc_value_new ();
	  sp_rpc_value_struct_set_key (value, tmp, v);
	  sp_rpc_value_unref (v);
	}

      if (rec > 2)
	do_set_value (v, rec - 1);
    }
}

static void
do_get_value (SPRpcValue * value, gint rec)
{
  gchar tmp[1024];

  switch (sp_rpc_value_get_type (value))
    {
    case SP_RPC_TYPE_NULL:
      do_print (rec, "NULL");
      break;

    case SP_RPC_TYPE_NUMBER:
      snprintf (tmp, sizeof (tmp), "N: %f", sp_rpc_value_get_number (value));
      do_print (rec, tmp);
      break;

    case SP_RPC_TYPE_BOOLEAN:
      snprintf (tmp, sizeof (tmp), "B: %s",
		sp_rpc_value_get_boolean (value) == TRUE ? "true" : "false");
      do_print (rec, tmp);
      break;

    case SP_RPC_TYPE_STRING:
      {
	gchar *tmp = sp_rpc_value_get_string (value);
	do_print (rec, tmp);
	g_free (tmp);
      }
      break;

    case SP_RPC_TYPE_ARRAY:
      do_print (rec, "A");
      do_get_array (value, rec + 1);
      break;

    case SP_RPC_TYPE_STRUCT:
      do_print (rec, "ST");
      do_get_struct (value, rec + 1);
      break;
    }
}

static void
do_get_array (SPRpcValue * value, gint rec)
{
  gint i, numb;

  numb = sp_rpc_value_array_length (value);
  for (i = 0; i < numb; i++)
    {
      SPRpcValue *v = sp_rpc_value_array_get (value, i);
      do_get_value (v, rec);
    }
}

static void
do_get_struct (SPRpcValue * value, gint rec)
{
  SPRpcValue *v;
  SPRpcIter iter;

  sp_rpc_value_struct_iter_init (value, &iter);
  while (sp_rpc_value_struct_iter_next (&iter, NULL, &v) == TRUE)
    do_get_value (v, rec);
}

static void
do_print (gint ret, gchar * str)
{
  for (; ret; ret--)
    printf ("  ");

  puts (str);
}
